@@ -5,7 +5,6 @@ typedef struct Socket Socket;
 typedef struct SocketPeer SocketPeer;
 
 #include "mount.h"
-#include "service.h"
 #include "socket-util.h"
 #include "unit.h"
 
@@ -16,7 +15,7 @@ typedef enum SocketExecCommand {
         SOCKET_EXEC_STOP_PRE,
         SOCKET_EXEC_STOP_POST,
         _SOCKET_EXEC_COMMAND_MAX,
-        _SOCKET_EXEC_COMMAND_INVALID = -1
+        _SOCKET_EXEC_COMMAND_INVALID = -EINVAL,
 } SocketExecCommand;
 
 typedef enum SocketType {
@@ -26,7 +25,7 @@ typedef enum SocketType {
         SOCKET_MQUEUE,
         SOCKET_USB_FUNCTION,
         _SOCKET_TYPE_MAX,
-        _SOCKET_TYPE_INVALID = -1
+        _SOCKET_TYPE_INVALID = -EINVAL,
 } SocketType;
 
 typedef enum SocketResult {
@@ -40,7 +39,7 @@ typedef enum SocketResult {
         SOCKET_FAILURE_TRIGGER_LIMIT_HIT,
         SOCKET_FAILURE_SERVICE_START_LIMIT_HIT,
         _SOCKET_RESULT_MAX,
-        _SOCKET_RESULT_INVALID = -1
+        _SOCKET_RESULT_INVALID = -EINVAL,
 } SocketResult;
 
 typedef struct SocketPort {
@@ -63,7 +62,7 @@ typedef enum SocketTimestamping {
         SOCKET_TIMESTAMPING_US,  /* SO_TIMESTAMP */
         SOCKET_TIMESTAMPING_NS,  /* SO_TIMESTAMPNS */
         _SOCKET_TIMESTAMPING_MAX,
-        _SOCKET_TIMESTAMPING_INVALID = -1,
+        _SOCKET_TIMESTAMPING_INVALID = -EINVAL,
 } SocketTimestamping;
 
 struct Socket {
@@ -174,6 +173,9 @@ int socket_collect_fds(Socket *s, int **fds);
 /* Called from the service code when a per-connection service ended */
 void socket_connection_unref(Socket *s);
 
+SocketPort *socket_port_free(SocketPort *p);
+DEFINE_TRIVIAL_CLEANUP_FUNC(SocketPort*, socket_port_free);
+
 void socket_free_ports(Socket *s);
 
 int socket_load_service_unit(Socket *s, int cfd, Unit **ret);
