@@ -215,17 +215,17 @@ static int run(const char *dest, const char *dest_early, const char *dest_late)
                         return r;
         }
 
-        /* Automatically add in a serial getty on the first virtualizer console */
+        /* Automatically add a serial getty to each available virtualizer console. */
         FOREACH_STRING(j,
                        "hvc0",
                        "xvc0",
                        "hvsi0",
                        "sclp_line0",
                        "ttysclp0",
-                       "3270!tty1") {
+                       "3270/tty1") {
                 _cleanup_free_ char *p = NULL;
 
-                p = path_join("/sys/class/tty", j);
+                p = path_join("/dev", j);
                 if (!p)
                         return -ENOMEM;
                 if (access(p, F_OK) < 0)
