@@ -10,13 +10,13 @@
 #include "generator.h"
 #include "log.h"
 #include "main-func.h"
-#include "mkdir.h"
+#include "mkdir-label.h"
 #include "proc-cmdline.h"
 #include "special.h"
 #include "string-util.h"
 #include "unit-name.h"
 
-static const char *arg_dest = "/tmp";
+static const char *arg_dest = NULL;
 static char *arg_resume_device = NULL;
 static char *arg_resume_options = NULL;
 static char *arg_root_options = NULL;
@@ -45,7 +45,7 @@ static int parse_proc_cmdline_item(const char *key, const char *value, void *dat
                 if (proc_cmdline_value_missing(key, value))
                         return 0;
 
-                if (!strextend_with_separator(&arg_resume_options, ",", value, NULL))
+                if (!strextend_with_separator(&arg_resume_options, ",", value))
                         return log_oom();
 
         } else if (streq(key, "rootflags")) {
@@ -53,7 +53,7 @@ static int parse_proc_cmdline_item(const char *key, const char *value, void *dat
                 if (proc_cmdline_value_missing(key, value))
                         return 0;
 
-                if (!strextend_with_separator(&arg_root_options, ",", value, NULL))
+                if (!strextend_with_separator(&arg_root_options, ",", value))
                         return log_oom();
 
         } else if (streq(key, "noresume")) {
@@ -84,8 +84,8 @@ static int process_resume(void) {
         if (!lnk)
                 return log_oom();
 
-        mkdir_parents_label(lnk, 0755);
-        if (symlink(SYSTEM_DATA_UNIT_PATH "/systemd-hibernate-resume@.service", lnk) < 0)
+        (void) mkdir_parents_label(lnk, 0755);
+        if (symlink(SYSTEM_DATA_UNIT_DIR "/systemd-hibernate-resume@.service", lnk) < 0)
                 return log_error_errno(errno, "Failed to create symlink %s: %m", lnk);
 
         r = unit_name_from_path(arg_resume_device, ".device", &device_unit);
@@ -109,19 +109,12 @@ static int process_resume(void) {
         return 0;
 }
 
-static int run(int argc, char *argv[]) {
+static int run(const char *dest, const char *dest_early, const char *dest_late) {
         int r = 0;
 
-        log_setup_generator();
+        arg_dest = ASSERT_PTR(dest);
 
-        if (argc > 1 && argc != 4)
-                return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
-                                       "This program takes three or no arguments.");
-
-        if (argc > 1)
-                arg_dest = argv[1];
-
-        /* Don't even consider resuming outside of initramfs. */
+        /* Don't even consider resuming outside of initrd. */
         if (!in_initrd()) {
                 log_debug("Not running in an initrd, quitting.");
                 return 0;
@@ -139,4 +132,4 @@ static int run(int argc, char *argv[]) {
         return process_resume();
 }
 
-DEFINE_MAIN_FUNCTION(run);
+DEFINE_MAIN_GENERATOR_FUNCTION(run);
