@@ -56,7 +56,7 @@ static int raw_verify(const char *fn, const char *verification_key) {
         return r;
 }
 
-int main(int argc, char *argv[]) {
+static int run_test(int argc, char *argv[]) {
         _cleanup_(mmap_cache_unrefp) MMapCache *m = NULL;
         char t[] = "/var/tmp/journal-XXXXXX";
         unsigned n;
@@ -91,7 +91,7 @@ int main(int argc, char *argv[]) {
 
                 dual_timestamp_get(&ts);
 
-                assert_se(asprintf(&test, "RANDOM=%lu", random() % RANDOM_RANGE));
+                assert_se(asprintf(&test, "RANDOM=%li", random() % RANDOM_RANGE));
 
                 iovec = IOVEC_MAKE_STRING(test);
 
@@ -141,3 +141,13 @@ int main(int argc, char *argv[]) {
 
         return 0;
 }
+
+int main(int argc, char *argv[]) {
+        assert_se(setenv("SYSTEMD_JOURNAL_COMPACT", "0", 1) >= 0);
+        run_test(argc, argv);
+
+        assert_se(setenv("SYSTEMD_JOURNAL_COMPACT", "1", 1) >= 0);
+        run_test(argc, argv);
+
+        return 0;
+}
