@@ -9,18 +9,11 @@
 #include <sys/mman.h>
 #include <sys/prctl.h>
 
-/* When we include libgen.h because we need dirname() we immediately
- * undefine basename() since libgen.h defines it as a macro to the POSIX
- * version which is really broken. We prefer GNU basename(). */
-#include <libgen.h>
-#undef basename
-
 #include "alloc-util.h"
 #include "bus-internal.h"
 #include "bus-kernel.h"
 #include "bus-label.h"
 #include "bus-message.h"
-#include "bus-util.h"
 #include "capability-util.h"
 #include "fd-util.h"
 #include "fileio.h"
@@ -41,10 +34,8 @@ void close_and_munmap(int fd, void *address, size_t size) {
 }
 
 void bus_flush_memfd(sd_bus *b) {
-        unsigned i;
-
         assert(b);
 
-        for (i = 0; i < b->n_memfd_cache; i++)
+        for (unsigned i = 0; i < b->n_memfd_cache; i++)
                 close_and_munmap(b->memfd_cache[i].fd, b->memfd_cache[i].address, b->memfd_cache[i].mapped);
 }
