@@ -1269,10 +1269,20 @@ SessionState session_get_state(Session *s) {
 int session_kill(Session *s, KillWho who, int signo) {
         assert(s);
 
-        if (!s->scope)
-                return -ESRCH;
+        switch (who) {
 
-        return manager_kill_unit(s->manager, s->scope, who, signo, NULL);
+        case KILL_ALL:
+                if (!s->scope)
+                        return -ESRCH;
+
+                return manager_kill_unit(s->manager, s->scope, KILL_ALL, signo, NULL);
+
+        case KILL_LEADER:
+                return RET_NERRNO(kill(s->leader, signo));
+
+        default:
+                assert_not_reached();
+        }
 }
 
 static int session_open_vt(Session *s) {
