@@ -111,10 +111,11 @@ int suggest_passwords(void) {
         int r;
 
         r = pwq_allocate_context(&pwq);
-        if (ERRNO_IS_NOT_SUPPORTED(r))
-                return 0;
-        if (r < 0)
+        if (r < 0) {
+                if (ERRNO_IS_NOT_SUPPORTED(r))
+                        return 0;
                 return log_error_errno(r, "Failed to allocate libpwquality context: %m");
+        }
 
         suggestions = new0(char*, N_SUGGESTIONS+1);
         if (!suggestions)
@@ -131,7 +132,7 @@ int suggest_passwords(void) {
         if (!joined)
                 return log_oom();
 
-        log_info("Password suggestions: %s", joined);
+        printf("Password suggestions: %s\n", joined);
         return 1;
 }
 
@@ -144,8 +145,6 @@ int quality_check_password(const char *password, const char *username, char **re
         assert(password);
 
         r = pwq_allocate_context(&pwq);
-        if (ERRNO_IS_NOT_SUPPORTED(r))
-                return 0;
         if (r < 0)
                 return log_debug_errno(r, "Failed to allocate libpwquality context: %m");
 
