@@ -306,7 +306,7 @@ MountAPIVFS=yes
 PrivateTmp=yes
 ExecStart=/bin/sh -c ' \\
     systemd-notify --ready; \\
-    while [[ ! -f /tmp/img/usr/lib/os-release ]] || ! grep -q -F MARKER /tmp/img/usr/lib/os-release; do \\
+    while [ ! -f /tmp/img/usr/lib/os-release ] || ! grep -q -F MARKER /tmp/img/usr/lib/os-release; do \\
         sleep 0.1; \\
     done; \\
     mount; \\
